from typing import Any

from django.urls import reverse
from django.views.generic.base import TemplateView
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin

from ..view_mixins import EdcLabelViewMixin


class HomeView(EdcViewMixin, NavbarViewMixin, EdcLabelViewMixin, TemplateView):
    template_name = "edc_label/home.html"
    navbar_name = "edc_label"
    navbar_selected_item = "label"

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        printer_setup_url = reverse("edc_label:printer_setup_url")
        kwargs.update(
            printer_setup_url=printer_setup_url,
        )
        return super().get_context_data(**kwargs)



# Generated by Django 4.2.7 on 2023-12-04 02:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_label", "0004_alter_zpllabeltemplates_options_and_more"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="zpllabeltemplates",
            name="edc_label_z_modifie_79a6b3_idx",
        ),
        migrations.AddIndex(
            model_name="zpllabeltemplates",
            index=models.Index(
                fields=["modified", "created"], name="edc_label_z_modifie_038fe9_idx"
            ),
        ),
    ]



import os
from string import Template

from django.apps import apps as django_apps
from django.contrib.staticfiles import finders
from django.core.exceptions import ObjectDoesNotExist


class LabelTemplateError(Exception):
    pass


class LabelTemplate:
    template_name = None

    def __init__(self, template_name=None, static_files_path=None):
        self.template_name = template_name or self.template_name
        try:
            zpl_label_template = django_apps.get_model(
                "edc_label.zpllabeltemplates"
            ).objects.get(name=self.template_name)
        except ObjectDoesNotExist:
            if static_files_path:
                path = finders.find(os.path.join(static_files_path, template_name))
            else:
                app_config = django_apps.get_app_config("edc_label")
                try:
                    path = app_config.label_templates[template_name]
                except KeyError:
                    raise LabelTemplateError(
                        f"Invalid label template name. "
                        f"Expected one of {list(app_config.label_templates.keys())}. "
                        f"Got '{template_name}'. "
                        f"See edc_label.app_config."
                    )
            if not os.path.exists(path or ""):
                raise LabelTemplateError(
                    f"Invalid label template path. "
                    f"Looking for  template '{template_name}'. "
                    f"Got {path}. See edc_label.app_config."
                )
            with open(path, "r") as f:
                self.template = f.read()
        else:
            self.template = zpl_label_template.zpl_data.strip()

    def __str__(self):
        return self.template_name

    def render(self, context):
        zpl_string = Template(self.template).safe_substitute(context)
        return zpl_string.replace("\n", "").replace("\r", "")



from typing import Optional

from django.utils import timezone

from .label_template import LabelTemplate


class Label:
    """A class that prepares data for x copies of labels."""

    label_name = "label"
    label_template_cls = LabelTemplate
    label_template_name = None

    def __init__(
        self,
        label_template_name: Optional[str] = None,
        static_files_path: Optional[str] = None,
    ):
        if label_template_name:
            self.label_template_name = label_template_name
        self.messages = None
        self.label_template = self.label_template_cls(
            template_name=self.label_template_name, static_files_path=static_files_path
        )

    def __str__(self):
        return f"{self.label_template_name}."

    @property
    def label_context(self):
        return {}

    def render_as_zpl_data(self, copies=None, context=None, encoding=None):
        copies = copies or 1
        timestamp = timezone.now().strftime("%Y-%m-%d %H:%M")
        encoding = "utf8" if encoding is None else encoding
        zpl_data = []
        for i in range(copies, 0, -1):
            context = context or self.label_context
            context.update(
                {"label_count": i, "label_count_total": copies, "timestamp": timestamp}
            )
            zpl = self.label_template.render(context)
            if encoding:
                zpl = zpl.encode(encoding)
            zpl_data.append(zpl)
        return (b"" if encoding else "").join(zpl_data)
